### standard imports ###
import numpy as np
import matplotlib.pyplot as plt
import time

### import MiM plus required instruments ###
from moku.instruments import MultiInstrument
from moku.instruments import Phasemeter
from moku.instruments import WaveformGenerator
from moku.instruments import Oscilloscope

### Establish connection and number of slots being used ###
mim = MultiInstrument('10.0.0.245', force_connect=True, platform_id=4)

idn = mim.serial_number()
prop = mim.describe()
print('Connected to ' + prop['hardware'] + ', S/N ' + idn)
print('Firmware version ' + str(prop['firmware']))

### Define instrument in each slot ###
wg = mim.set_instrument(1, WaveformGenerator)
phm = mim.set_instrument(2, Phasemeter)
osc = mim.set_instrument(3, Oscilloscope)

print('Instruments Set')

### Define connection map ###

connections = [dict(source="Slot1OutA", destination="Slot2InA"),
               dict(source="Slot1OutA", destination="Slot3InA"),
               dict(source="Slot2OutA", destination="Slot3InB"),
               ]

mim.set_connections(connections=connections)

print('Connections Set')

### Set up Frequency Source

wg.generate_waveform(channel=1, type='Sine', amplitude=1, frequency=100.0e6)

### Set up Phasemeter ###
phm.generate_output(1, signal='Sine', amplitude=1, phase_locked=True, frequency_multiplier=1)
phm.set_pm_loop(1,auto_acquire=False,frequency=100e6,bandwidth='100kHz')
phm.sync_output_phase()

### set the timebase for the oscilloscope ### 
osc.set_timebase(-25e-9, 25e-9)

print('oscillator configured')

### Let's plot ### 

plt.ion()
fig = plt.figure()
ax = fig.add_subplot(111)
ax.set_xlabel('Time (s)')
ax.set_ylabel('Voltage (V)')
data = osc.get_data()
line1, = ax.plot(data['time'], data['ch1'], 'r', label='ChA')
line1.set_label('ChA') 
line2, = ax.plot(data['time'], data['ch2'], 'b', label='ChB')
line2.set_label('ChB')  
ax.legend(loc='upper right')

for x in range(0,500):
    data = osc.get_data()
    line1.set_ydata(data['ch1'])
    line2.set_ydata(data['ch2'])
    fig.canvas.draw()
    fig.canvas.flush_events()
    time.sleep(0.1)

mim.relinquish_ownership()